@@ -888,7 +888,7 @@ Uwaga: Aby ignorowa wielko liter dla jednego wyszukiwania: /ignore\c<ENTER>
 
   1. Pocztek edycji pliku "vimrc" zaley od Twojego systemu:
      :edit ~/.vimrc	     dla Uniksa
-     :edit $VIM/_vimrc       dla MS-Windows
+     :edit ~/_vimrc          dla MS-Windows
   2. Teraz wczytaj przykadowy plik "vimrc":
      :read $VIMRUNTIME/vimrc_example.vim
   3. Zapisz plik:
@@ -954,7 +954,7 @@ UWAGA: Uzupenianie dziaa dla wielu polece. Sprbuj wcisn CTRL-D i <TAB>.
 	Wydawca: New Riders
   Pierwsza ksika cakowicie powicona Vimowi. Uyteczna zwaszcza dla
   pocztkujcych. Zawiera wiele przykadw i ilustracji.
-  Zobacz http://iccf-holland.org./click5.html
+  Zobacz https://iccf-holland.org./click5.html
 
   Starsza pozycja i bardziej o Vi ni o Vimie, ale take warta
   polecenia:
