@@ -18,6 +18,7 @@ static int pum_selected;		// index of selected item or -1
 static int pum_first = 0;		// index of top item
 
 static int call_update_screen = FALSE;
+static int pum_in_cmdline = FALSE;
 
 static int pum_height;			// nr of displayed pum items
 static int pum_width;			// width of displayed pum items
@@ -834,7 +835,6 @@ pum_set_selected(int n, int repeat UNUSED)
 	{
 	    win_T	*curwin_save = curwin;
 	    tabpage_T   *curtab_save = curtab;
-	    int		res = OK;
 # ifdef FEAT_PROP_POPUP
 	    use_popup_T	use_popup;
 # else
@@ -864,7 +864,8 @@ pum_set_selected(int n, int repeat UNUSED)
 	    ++no_u_sync;
 	    resized = prepare_tagpreview(FALSE, FALSE, use_popup);
 	    --no_u_sync;
-	    --RedrawingDisabled;
+	    if (RedrawingDisabled > 0)
+		--RedrawingDisabled;
 	    g_do_tagpreview = 0;
 
 	    if (curwin->w_p_pvw
@@ -873,6 +874,8 @@ pum_set_selected(int n, int repeat UNUSED)
 # endif
 		    )
 	    {
+		int	res = OK;
+
 		if (!resized
 			&& curbuf->b_nwindows == 1
 			&& curbuf->b_fname == NULL
@@ -918,13 +921,10 @@ pum_set_selected(int n, int repeat UNUSED)
 			    ml_append(lnum++, p, 0, FALSE);
 			    break;
 			}
-			else
-			{
-			    *e = NUL;
-			    ml_append(lnum++, p, (int)(e - p + 1), FALSE);
-			    *e = '\n';
-			    p = e + 1;
-			}
+			*e = NUL;
+			ml_append(lnum++, p, (int)(e - p + 1), FALSE);
+			*e = '\n';
+			p = e + 1;
 		    }
 		    // delete the empty last line
 		    ml_delete(curbuf->b_ml.ml_line_count);
@@ -1067,6 +1067,11 @@ pum_undisplay(void)
     pum_array = NULL;
     redraw_all_later(UPD_NOT_VALID);
     redraw_tabline = TRUE;
+    if (pum_in_cmdline)
+    {
+	clear_cmdline = TRUE;
+	pum_in_cmdline = FALSE;
+    }
     status_redraw_all();
 #if defined(FEAT_PROP_POPUP) && defined(FEAT_QUICKFIX)
     // hide any popup info window
@@ -1193,6 +1198,8 @@ pum_position_at_mouse(int min_width)
 	pum_row = mouse_row + 1;
 	if (pum_height > Rows - pum_row)
 	    pum_height = Rows - pum_row;
+	if (pum_row + pum_height > cmdline_row)
+	    pum_in_cmdline = TRUE;
     }
     else
     {
@@ -1518,7 +1525,7 @@ pum_show_popupmenu(vimmenu_T *menu)
     // pum_size being zero.
     if (pum_size <= 0)
     {
-	emsg(e_menu_only_exists_in_another_mode);
+	emsg(_(e_menu_only_exists_in_another_mode));
 	return;
     }
 
@@ -1641,7 +1648,7 @@ pum_make_popup(char_u *path_name, int use_mouse_pos)
     {
 	// Hack: set mouse position at the cursor so that the menu pops up
 	// around there.
-	mouse_row = curwin->w_winrow + curwin->w_wrow;
+	mouse_row = W_WINROW(curwin) + curwin->w_wrow;
 	mouse_col = curwin->w_wincol + curwin->w_wcol;
     }
 
